@@ -1,5 +1,7 @@
+from __future__ import annotations
+
 # coding=utf-8
-from ada.concepts.connections import Bolts, JointBase, Weld
+from ada.concepts.connections import Bolts, Weld
 from ada.concepts.curves import ArcSegment, CurvePoly, CurveRevolve, LineSegment
 from ada.concepts.levels import Assembly, Part
 from ada.concepts.piping import Pipe, PipeSegElbow, PipeSegStraight
@@ -14,8 +16,10 @@ from ada.concepts.primitives import (
     PrimSweep,
     Shape,
 )
-from ada.concepts.structural import Beam, Material, Plate, Section, Wall
+from ada.concepts.structural import Beam, Plate, Wall
 from ada.config import User
+from ada.materials import Material
+from ada.sections import Section
 
 __author__ = "Kristoffer H. Andersen"
 __all__ = [
@@ -44,6 +48,5 @@ __all__ = [
     "ArcSegment",
     "User",
     "Bolts",
-    "JointBase",
     "Weld",
 ]
