@@ -1,18 +1,22 @@
+from __future__ import annotations
+
 import logging
 import reprlib
 from bisect import bisect_left, bisect_right
 from itertools import chain
 from operator import attrgetter
-from typing import Iterable
+from typing import Iterable, List, Union
 
 import numpy as np
 import toolz
 from pyquaternion import Quaternion
 
 from ada.config import Settings
+from ada.core.utils import Counter, points_in_cylinder, roundoff, vector_length
+from ada.materials import Material
 
-from ..core.utils import Counter, points_in_cylinder, roundoff, vector_length
-from .structural import Beam, Material, Plate, Section
+from .points import Node
+from .structural import Beam, Plate, Section
 
 __all__ = [
     "Nodes",
@@ -36,15 +40,9 @@ class BaseCollections:
 
 
 class Beams(BaseCollections):
-    """
-    A collections of Beam objects
+    """A collections of Beam objects"""
 
-    :param beams:
-    :param unique_ids:
-    :param parent:
-    """
-
-    def __init__(self, beams=None, unique_ids=True, parent=None):
+    def __init__(self, beams: Iterable[Beam] = None, unique_ids=True, parent=None):
 
         super().__init__(parent)
         beams = [] if beams is None else beams
@@ -257,23 +255,13 @@ class Plates(BaseCollections):
         """
         return self._dmap
 
-    def _add_node_to_global(self, pnt):
-        """
-
-        :param pnt:
-        :type pnt: ada.Node
-        """
+    def _add_node_to_global(self, pnt: Node):
         old_node = self._parent.nodes.add(pnt)
         if old_node is not None:
             pnt = old_node
         return pnt
 
-    def add(self, plate):
-        """
-
-        :param plate:
-        :type plate: ada.Plate
-        """
+    def add(self, plate: Plate):
         if plate.name is None:
             raise Exception("Name is not allowed to be None.")
 
@@ -374,7 +362,7 @@ class Connections(BaseCollections):
         :param joint_func: Pass a function for mapping the generic Connection classes to a specific reinforced Joints
         :param point_tol:
         """
-        from ada import JointBase
+        from ada.concepts.connections import JointBase
         from ada.core.clash_check import are_beams_connected
 
         ass = self._parent.get_assembly()
@@ -384,9 +372,7 @@ class Connections(BaseCollections):
         nmap = dict()
 
         for bm1_, beams_ in bm_res:
-            nmap_, nodes_ = are_beams_connected(bm1_, beams_, out_of_plane_tol, point_tol)
-            nmap.update(nmap_)
-            nodes += nodes_
+            are_beams_connected(bm1_, beams_, out_of_plane_tol, point_tol, nodes, nmap)
 
         for node, mem in nmap.items():
             if joint_func is not None:
@@ -400,15 +386,9 @@ class Connections(BaseCollections):
 
 
 class Materials(BaseCollections):
-    """
-    Collection of materials
+    """Collection of materials"""
 
-    :param materials:
-    :param unique_ids:
-    :param parent:
-    """
-
-    def __init__(self, materials=None, unique_ids=True, parent=None, units="m"):
+    def __init__(self, materials: Iterable[Material] = None, unique_ids=True, parent=None, units="m"):
         super().__init__(parent)
         self._materials = sorted(materials, key=attrgetter("name")) if materials is not None else []
         self._unique_ids = unique_ids
@@ -682,12 +662,12 @@ class Nodes:
         self._maxid = max(self._idmap.keys()) if len(self._nodes) > 0 else 0
         self._bbox = self._get_bbox() if len(self._nodes) > 0 else None
 
-    def renumber(self):
+    def renumber(self, start_id: int = 1):
         """
         Ensures that the node numberings starts at 1 and has no holes in its numbering.
 
         """
-        for i, n in enumerate(sorted(self._nodes, key=attrgetter("id")), start=1):
+        for i, n in enumerate(sorted(self._nodes, key=attrgetter("id")), start=start_id):
             if i != n.id:
                 n.id = i
 
@@ -929,11 +909,10 @@ class Nodes:
         insert_node(node, index)
         return node
 
-    def remove(self, nodes):
+    def remove(self, nodes: Union[Node, List[Node]]):
         """
         Remove node from the nodes container
         :param nodes: Node-object to be removed
-        :type nodes: ada.Node or List[ada.Node]
         :return:
         """
         from collections.abc import Iterable
